%{
  #include <stdio.h>  
  extern char* yytext;
  int yylex(void);

  void yyerror(char *);
  int cost(int);
  enum { STORE, ADD, SUB, MULT, DIV, LOADIMM, LOAD };
  int cost_factors[] = { 2, 1, 1, 16, 32, 1, 2 };  
%}

%token ident number

%%

Start : Block               { printf("Cost: %d\n", $$); }
Block : Block Assign        { $$ = $1 + $2; }
      | Assign              { $$ = $1; }
Assign: ident '=' Expr      { $$ = cost(STORE) + $3; }
Expr  : Expr '+' Term       { $$ = $1 + cost(ADD) + $3; }
      | Expr '-' Term       { $$ = $1 + cost(SUB) + $3; }
      | Term                { $$ = $1; }
Term  : Term '*' Factor     { $$ = $1 + cost(MULT) + $3; }
      | Term '/' Factor     { $$ = $1 + cost(DIV) + $3; }
      | Factor              { $$ = $1; }
Factor: '(' Expr ')'        { $$ = $2; }
      | number              { $$ = cost(LOADIMM); }
      | ident               { /* int i = hash(ident);
                              if (table[i].loaded == 0) { */
                                $$ = $$ + cost(LOAD);
                              /*  table[i].loaded = 1; 
                              }
                              else $$ = 0; */
                            }

%%

int cost(int token) {
  return cost_factors[token];
}

void yyerror(char *s) {
  printf("%s: %s\n", s, yytext);
}

int main(void) {
  yyparse();
}

