%{
  #include <stdio.h>  
  extern char* yytext;
  int yylex(void);

  void yyerror(char *);
  void Emit(char *, int, int, int);
  void EmitLD(char *, int, char *);
  void EmitLI(char *, int, int);
  int NxReg(void);
%}

%token ident number

%%

Start : Expr                { $$ = $1; }
Expr  : Expr '+' Term       { $$=NxReg(); Emit("add", $$, $1, $3); }
      | Expr '-' Term       { $$=NxReg(); Emit("sub", $$, $1, $3); }
      | Term                { $$ = $1; }
Term  : Term '*' Factor     { $$=NxReg(); Emit("mul", $$, $1, $3); }
      | Term '/' Factor     { $$=NxReg(); Emit("div", $$, $1, $3); }
      | Factor              { $$ = $1; }
Factor: '(' Expr ')'        { $$ = $2; }
      | number              { $$=NxReg(); EmitLI("mov", $$, yylval); }
      | ident               { $$=NxReg(); EmitLD("ldr", $$, yytext); }

%%

int NxReg(void) {
  static int reg = 0;
  if (reg > 5) { reg = 0; return reg; }
  return reg++;
}

void EmitLD(char *op, int rd, char *adr) {
  printf("\tldr r%d, =%s\n", rd, adr);
  printf("\t%s r%d, [r%d]\n", op, rd, rd);
}

void EmitLI(char *op, int rd, int val) {
  printf("\t%s r%d, #%d\n", op, rd, val);
}

void Emit(char *op, int rd, int rs1, int rs2) {
  printf("\t%s r%d, r%d, r%d\n", op, rd, rs1, rs2);
}

void yyerror(char *s) {
  printf("%s: %s\n", s, yytext);
}

int main(void) {
  yyparse();
}

